% This syntax is used to plot simulations for Bose et al.'s state-2 and -3
% experiments (Bose et al., 2003)

% Copyright: 
% Fan Wu and Daniel. A Beard, May 2007
% Computational Engineering Group
% Biotechnology and Bioengineering Center
% Medical College of Wisconsin
 
%% Defining global variables
clear; clear global;

global iH_x idPsi iATP_x iADP_x iAMP_x iGTP_x iGDP_x iPi_x iNADH_x ...
       iQH2_x iOAA_x iACCOA_x iCIT_x iICIT_x iAKG_x iSCOA_x iCOASH_x ...
       iSUC_x iFUM_x iMAL_x iGLU_x iASP_x iK_x iMg_x iO2_x iCO2tot_x ...
       iCred_i iATP_i iADP_i iAMP_i iPi_i iH_i iMg_i iK_i ...
       iATP_c iADP_c iPi_c iH_c iMg_c iK_c ...
       iPYR_x iPYR_i iPYR_c iCIT_i iCIT_c iAKG_i iAKG_c iSUC_i iSUC_c ...
       iMAL_i iMAL_c iASP_i iASP_c iGLU_i iGLU_c iFUM_i iFUM_c ...
       iICIT_i iICIT_c iGLC_c iG6P_c iPCr_c iAMP_c;

define_global();

%% Setting physical parameters and fixed pools

W_m = 0.664*1.09;   % mitochondrial water space (ml water per ml mito) [VB2002]
W_x = 0.9*W_m;      % Matrix water space, (l mito water) (l mito)^{-1}
W_i = 0.1*W_m;      % IM water space, (l IM water) (l mito)^{-1} 
W_c = 80;           % buffer, (l buffer water) (l mito)^{-1}
rho_m = 3.6697e-6;  % (l mito) (mg protein)^{-1}

Ctot   = 2.70e-3;               % M; total cytoC
Qtot   = 1.35e-3;               % M; total Q + QH2
NADtot = 2.97e-3;               % M; total NAD+NADH
Atot   = 10e-3;                 % M; total Adenosine


%% Loading simulation results
% state-2
load B2PI.mat; 
H_x_PI_r = H_x_PI;
NADH_x_PI_r = NADH_x_PI;
Cred_i_PI_r = Cred_i_PI;
dPsi_PI_r = dPsi_PI;
ATP_PI_r = ATP_x_PI;

% state-3
load B3PI.mat;
H_x_PI_a = H_x_PI;
NADH_x_PI_a = NADH_x_PI;
Cred_i_PI_a = Cred_i_PI;
dPsi_PI_a = dPsi_PI;
ATP_PI_a = ATP_x_PI;


%% Loading state-2 experimental data curves (from Bose et al., 2003)
load Pi_exp_r; Pi_exp_r = Pi_exp_r/1e3;
load d_Psi_exp_r;
load NADH_exp_r;
load Pi_exp_a;
load JO2_exp_a;

% Cytochrome C data--percent reduced (from Bose et al., 2003)
CPR_exp_r(1) = 0.092;
CPR_exp_r(2) = 0.105;


%% Loading state-3 experimental data curves (from Bose et al., 2003)
load Pi_exp_a; Pi_exp_a = Pi_exp_a/1e3;
load d_Psi_exp_a;
load NADH_exp_a;

% Cytochrome C data--percent reduced (from Bose et al., 2003)
CPR_exp_a(1) = 0.084;
CPR_exp_a(2) = 0.171;


%% Plotting the results

figure(401); clf;
axes('position',[0.15 0.15 0.75 0.75]);
box on; hold on;
plot(Pi_r*1e3,NADH_x_PI_r/NADtot,'--k','LineWidth',2);
plot(Pi_exp_r*1e3,NADH_exp_r,'ko','MarkerSize',12,'MarkerFaceColor',[1 1 1]);
plot(Pi_a*1e3,NADH_x_PI_a/NADtot,'-k','LineWidth',2);
plot(Pi_exp_a*1e3,NADH_exp_a,'k^','MarkerSize',12,'MarkerFaceColor',[0.6 0.6 0.6]);
set(gca,'Fontsize',18);
set(gca,'Xtick',0:2:10);
set(gca,'Ytick',0:0.2:1);
set(gca,'XtickLabel',[]);
% xlabel('Buffer [PI] (mM)');
ylabel('NADH (Normalized)');
xlim([0 10]);
ylim([0 1.0]);
text((10-0)*0.05,(1-0)*0.9,'A','FontSize',24);
text((10-0)*0.35,(1-0)*0.88,'[ADP]_{buffer} = 0 mM','FontSize',16);
text((10-0)*0.35,(1-0)*0.44,'[ADP]_{buffer} = 1.3 mM','FontSize',16);
print -f401 -r600 -depsc Bose_NADH;

figure(402); clf;
axes('position',[0.15 0.15 0.75 0.75]);
box on; hold on;
plot(Pi_r*1e3,JO2_r,'--k','LineWidth',2);
plot([0 3],[30.8 40],'ko','MarkerSize',12,'MarkerFaceColor',[1 1 1]);
plot(Pi_a*1e3,JO2_a,'-k','LineWidth',2);
plot(Pi_exp_a*1e3,JO2_exp_a,'k^','MarkerSize',12,'MarkerFaceColor',[0.6 0.6 0.6]);
set(gca,'Fontsize',18);
set(gca,'Xtick',0:2:10);
set(gca,'Ytick',0:50:350);
set(gca,'XtickLabel',[]);
% xlabel('Buffer [PI] (mM)')
ylabel('MV_{O_2} (mol O_2 min^{-1} (mol Cyto A)^{-1})');
xlim([0 10]);
ylim([0 350]);
text((10-0)*0.05,(350-0)*0.9,'B','FontSize',24);
text((10-0)*0.35,(350-0)*0.23,'[ADP]_{buffer} = 0 mM','FontSize',16);
text((10-0)*0.35,(350-0)*0.67,'[ADP]_{buffer} = 1.3 mM','FontSize',16);
print -f402 -r600 -depsc Bose_MVO2;

figure(403); clf;
axes('position',[0.15 0.15 0.75 0.75]);
box on;hold on;
plot(Pi_r*1e3,Cred_i_PI_r/Ctot,'--k','LineWidth',2);
plot([0 3],CPR_exp_r,'ko','MarkerSize',12,'MarkerFaceColor',[1 1 1]);
plot(Pi_a*1e3,Cred_i_PI_a/Ctot,'-k','LineWidth',2);
plot([0 3],CPR_exp_a,'k^','MarkerSize',12,'MarkerFaceColor',[0.6 0.6 0.6]);
set(gca,'Fontsize',18);
set(gca,'Xtick',0:2:10);
set(gca,'Ytick',0:0.05:0.25);
set(gca,'XtickLabel',[]);
% xlabel('Buffer [PI] (mM)');
ylabel('CytoC^{2+} (Normalized)');
xlim([0 10]);
ylim([0 0.25]);
text((10-0)*0.05,(0.25-0)*0.9,'C','FontSize',24);
text((10-0)*0.35,(0.25-0)*0.53,'[ADP]_{buffer} = 0 mM','FontSize',16);
text((10-0)*0.35,(0.25-0)*0.8,'[ADP]_{buffer} = 1.3 mM','FontSize',16);
print -f403 -r600 -depsc Bose_CPR;

figure(404); clf;
axes('position',[0.15 0.15 0.75 0.75]);
box on;hold on;
p = plot(Pi_r*1e3,-log10(H_x_PI_r),'--k','LineWidth',2);
plot([0 3],[7.14 7.13],'ko','MarkerSize',12,'MarkerFaceColor',[1 1 1]);
p = plot(Pi_a*1e3,-log10(H_x_PI_a),'-k','LineWidth',2);
plot([0 3],[7.16 7.10],'k^','MarkerSize',12,'MarkerFaceColor',[0.6 0.6 0.6]);
set(gca,'Fontsize',18);
set(gca,'Xtick',0:2:10);
set(gca,'Ytick',6.5:0.25:7.5);
set(gca,'XtickLabel',[]);
% xlabel('Buffer [PI] (mM)');
ylabel('Matrix pH');
xlim([0 10]);
ylim([6.5 7.5]);
text((10-0)*0.05,(7.5-6.5)*0.9+6.5,'D','FontSize',24);
text((10-0)*0.35,(7.5-6.5)*0.5+6.5,'[ADP]_{buffer} = 0 mM','FontSize',16);
text((10-0)*0.35,(7.5-6.5)*0.7+6.5,'[ADP]_{buffer} = 1.3 mM','FontSize',16);
print -f404 -r600 -depsc Bose_pH;

figure(405); clf;
axes('position',[0.15 0.15 0.75 0.75]);
box on; hold on;
plot(Pi_r*1e3,dPsi_PI_r,'--k','LineWidth',2);
plot(Pi_exp_r*1e3,d_Psi_exp_r,'ko','MarkerSize',12,'MarkerFaceColor',[1 1 1]);
plot(Pi_a*1e3,dPsi_PI_a,'-k','LineWidth',2);
plot(Pi_exp_a*1e3,d_Psi_exp_a,'k^','MarkerSize',12,'MarkerFaceColor',[0.6 0.6 0.6]);
set(gca,'Fontsize',18);
set(gca,'Xtick',0:2:10);
set(gca,'Ytick',130:20:210);
% set(gca,'XtickLabel',[]);
xlabel('Buffer [PI] (mM)');
ylabel('\Delta\Psi (mV)');
xlim([0 10]);
ylim([130 215]);
text((10-0)*0.05,(215-130)*0.9+130,'E','FontSize',24);
text((10-0)*0.35,(215-130)*0.90+130,'[ADP]_{buffer} = 0 mM','FontSize',16);
text((10-0)*0.35,(215-130)*0.62+130,'[ADP]_{buffer} = 1.3 mM','FontSize',16);
print -f405 -r600 -depsc Bose_dPsi;

figure(406); clf;
axes('position',[0.15 0.15 0.75 0.75]);
box on; hold on;
plot(Pi_r*1e3,ATP_PI_r/Atot,'--k','LineWidth',2);
% plot(Pi_exp_r*1e3,d_Psi_exp_r,'ko','MarkerSize',12,'MarkerFaceColor',[0.6 0.6 0.6]);
plot(Pi_a*1e3,ATP_PI_a/Atot,'-k','LineWidth',2);
% plot(Pi_exp_a*1e3,d_Psi_exp_a,'k^','MarkerSize',12,'MarkerFaceColor',[0.6 0.6 0.6]);
set(gca,'Fontsize',18);
set(gca,'Xtick',0:2:10);
set(gca,'Ytick',0:0.2:1);
% set(gca,'XtickLabel',[]);
xlabel('Buffer [PI] (mM)');
ylabel('Matrix ATP/A_{tot}');
xlim([0 10]);
ylim([0 1]);
text((10-0)*0.05,(1-0)*0.9,'F','FontSize',24);
text((10-0)*0.35,(1-0)*0.82,'[ADP]_{buffer} = 0 mM','FontSize',16);
text((10-0)*0.35,(1-0)*0.33,'[ADP]_{buffer} = 1.3 mM','FontSize',16);
print -f406 -r600 -depsc Bose_ATPx;

display(' ');
display('done...yeah~');


%% End of the function