% This syntax is used to generate simulations for LaNoue et al.'s state-2 
% time courses on isolated rat heart mitochondria (LaNoue et al., 1970)

% Copyright: 
% Fan Wu and Daniel. A Beard, May 2007
% Computational Engineering Group
% Biotechnology and Bioengineering Center
% Medical College of Wisconsin
 
%% Defining global variables
clear; clear global;

global iH_x idPsi iATP_x iADP_x iAMP_x iGTP_x iGDP_x iPi_x iNADH_x ...
       iQH2_x iOAA_x iACCOA_x iCIT_x iICIT_x iAKG_x iSCOA_x iCOASH_x ...
       iSUC_x iFUM_x iMAL_x iGLU_x iASP_x iK_x iMg_x iO2_x iCO2tot_x ...
       iCred_i iATP_i iADP_i iAMP_i iPi_i iH_i iMg_i iK_i ...
       iATP_c iADP_c iPi_c iH_c iMg_c iK_c ...
       iPYR_x iPYR_i iPYR_c iCIT_i iCIT_c iAKG_i iAKG_c iSUC_i iSUC_c ...
       iMAL_i iMAL_c iASP_i iASP_c iGLU_i iGLU_c iFUM_i iFUM_c ...
       iICIT_i iICIT_c iGLC_c iG6P_c iPCr_c iAMP_c;

define_global();


%% Setting physical parameters and fixed pools
W_m = 0.664*1.09;   % mitochondrial water space (ml water per ml mito) [VB2002]
W_x = 0.9*W_m;      % Matrix water space, (l mito water) (l mito)^{-1}
W_i = 0.1*W_m;      % IM water space, (l IM water) (l mito)^{-1} 
W_c = 80;           % buffer, (l buffer water) (l mito)^{-1}
rho_m = 3.6697e-6;  % (l mito) (mg protein)^{-1}

Ctot   = 2.70e-3;               % M; total cytoC
Qtot   = 1.35e-3;               % M; total Q + QH2
NADtot = 2.97e-3;               % M; total NAD+NADH

% Defining the minimum concentration allowed for components
MinCon = 1e-32;   

%% Loading experimental data
load state4con.mat;
t_exp = state4con(:,1);

CT_exp = state4con(:,3);
AKG_exp = state4con(:,4);
SUC_exp = state4con(:,5);
MAL_exp = state4con(:,6);
FUM_exp = state4con(:,7);
PYR_uptake_exp = state4con(:,8);
% According to 2mM and 5mM initial concentration for Pyruvate and Malate in
% LaNoue et al's 1970 paper
PYR_exp = 2/5*MAL_exp(1) - PYR_uptake_exp; 

load fig7cit.mat;
t_exp = fig7cit(:,1);
CTtot_exp = fig7cit(:,2);
CTextm_exp = fig7cit(:,3);
CTmito_exp = fig7cit(:,5);

load fig3gluasp.mat;
t_exp_fig3 = fig3gluasp(:,1);
GLU_exp = fig3gluasp(:,4);
ASP_exp = fig3gluasp(:,5);

%% Loading model parameters and initial conditions

load param14.mat;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Part I: Simulating LaNoue et al.'s state-2 experiments with 2 mM pyruvate and 5 mM malate
% Defining initial conditions
xo = default_IC_vitro();

%% Adding PI and incubating for 30 seconds
% Buffer concentrations
xo(iH_c)    = 10^(-7.2); % LaNoue et al., 1970
xo(iMg_c)   = 5e-3;      % LaNoue et al., 1970
xo(iK_c)    = 150e-3;    % LaNoue et al., 1970
xo(iGLC_c)  = 30e-3;     % LaNoue et al., 1970
% PI and ADP concentrations (state-2)
xo(iPi_c)   = 20e-3;     % LaNoue et al., 1970
xo(iADP_c)  = 1e-9;

xo(3:end) = max(MinCon,xo(3:end));

indexC = [1 3:63];
options = odeset('MaxStep',5e-1,'NonNegative', indexC);

% tic
ExpType = 1;
StateType = 2;
t_ib = 30; % incubation time
[t,x] = ode15s(@IsoMito_dXdT,[0 t_ib],xo,options,param,ExpType,StateType);
% toc
xo = x(end,:);
x1 = x;
t1 = t;

%% Adding pyruvate and malate
xo(iPYR_c) = 2e-3;
xo(iMAL_c) = 5e-3;

%% Running simulations
tic
ExpType = 1;
StateType = 2;

[t,x] = ode15s(@IsoMito_dXdT,[0 500],xo,options,param,ExpType,StateType,0);
toc

x = [x1' x']';
t = [(t1-t_ib)' t']';


%% Computing simulated concentrations
CIT_mito  = ( x(:,iCIT_x)*W_x + x(:,iCIT_i)*W_i )*rho_m; 
CIT_extm  = x(:,iCIT_c)*W_c*rho_m;
ICIT_mito = ( x(:,iICIT_x)*W_x + x(:,iICIT_i)*W_i )*rho_m; 
ICIT_extm = x(:,iICIT_i)*W_c*rho_m;
CT_mito   = CIT_mito + ICIT_mito;
CT_extm   = CIT_extm + ICIT_extm;

AKG_mito = ( x(:,iAKG_x)*W_x + x(:,iAKG_i)*W_i )*rho_m; 
AKG_extm = x(:,iAKG_c)*W_c*rho_m;

SUC_mito = ( x(:,iSUC_x)*W_x + x(:,iSUC_i)*W_i )*rho_m; 
SUC_extm = x(:,iSUC_c)*W_c*rho_m;

MAL_mito = ( x(:,iMAL_x)*W_x + x(:,iMAL_i)*W_i )*rho_m; 
MAL_extm = x(:,iMAL_c)*W_c*rho_m;

FUM_mito = ( x(:,iFUM_x)*W_x + x(:,iFUM_i)*W_i )*rho_m; 
FUM_extm = x(:,iFUM_c)*W_c*rho_m; 

PYR_mito = ( x(:,iPYR_x)*W_x + x(:,iPYR_i)*W_i )*rho_m; 
PYR_extm = x(:,iPYR_c)*W_c*rho_m;

OAA_mito = x(:,iOAA_x)*W_x *rho_m; 

SCOA_mito = x(:,iSCOA_x)*W_x *rho_m; 

%% Plotting figures
% [PYR]
figure(201); clf;
axes('position',[0.15 0.15 0.75 0.75]);
box on; hold on;
plot(t((length(t1)+1):end),(PYR_mito((length(t1)+1):end)+PYR_extm((length(t1)+1):end))*1e9,'k','LineWidth',2.0);
p = plot(t_exp, PYR_exp, 'k^','MarkerFaceColor', [0.6 0.6 0.6],'MarkerSize',12);
set(gca,'Fontsize',18);
set(gca,'Ytick',0:200:800);
set(gca,'XtickLabel',[]);
% xlabel('Time (sec)');
ylabel('[PYR]_{total} (nmoles/mg)');
xlim([0 500]);
ylim([0 800]);
text((500-0)*0.05,(800-0)*0.9,'A','FontSize',24);
print -f201 -r600 -depsc L2_PYR;

% [CIT]
figure(202); clf;
axes('position',[0.15 0.15 0.75 0.75]);
box on; hold on;
plot(t((length(t1)+1):end),(CT_mito((length(t1)+1):end)+CT_extm((length(t1)+1):end))*1e9,'k-',...
     t((length(t1)+1):end),CT_extm((length(t1)+1):end)*1e9,'k--','LineWidth',2.0);
plot(t_exp,CTtot_exp,'k^','MarkerFaceColor', [0.6 0.6 0.6],'MarkerSize',12);
plot(t_exp,CTextm_exp,'ko','MarkerFaceColor', [1 1 1],'MarkerSize',12);
set(gca,'Fontsize',18);
set(gca,'Ytick',0:10:30);
set(gca,'XtickLabel',[]);
% xlabel('Time (sec)');
ylabel('[CIT]_{buffer} and [CIT]_{total} (nmoles/mg)');
xlim([0 500]);
ylim([0 30]);
text((500-0)*0.05,(30-0)*0.9,'B','FontSize',24);
print -f202 -r600 -depsc L2_CIT;

% [AKG]
figure(203); clf;
axes('position',[0.15 0.15 0.75 0.75]);
box on; hold on;
plot(t((length(t1)+1):end),(AKG_mito((length(t1)+1):end)+AKG_extm((length(t1)+1):end))*1e9,'k','LineWidth',2.0);
p = plot(t_exp, AKG_exp, 'k^','MarkerFaceColor', [0.6 0.6 0.6],'MarkerSize',12);
set(gca,'Fontsize',18);
set(gca,'Ytick',0:20:60);
set(gca,'XtickLabel',[]);
% xlabel('Time (sec)');
ylabel('[AKG]_{total} (nmoles/mg)');
xlim([0 500]);
ylim([0 60]);
text((500-0)*0.05,(60-0)*0.9,'C','FontSize',24);
print -f203 -r600 -depsc L2_AKG;

% [SUC]
figure(204); clf;
axes('position',[0.15 0.15 0.75 0.75]);
box on; hold on;
plot(t((length(t1)+1):end),(SUC_mito((length(t1)+1):end)+SUC_extm((length(t1)+1):end))*1e9,'k','LineWidth',2.0);
p = plot(t_exp, SUC_exp, 'k^','MarkerFaceColor', [0.6 0.6 0.6],'MarkerSize',12);
set(gca,'Fontsize',18);
set(gca,'Ytick',0:10:30);
set(gca,'XtickLabel',[]);
% xlabel('Time (sec)');
ylabel('[SUC]_{total} (nmoles/mg)');
xlim([0 500]);
ylim([0 30]);
text((500-0)*0.05,(30-0)*0.9,'D','FontSize',24);
print -f204 -r600 -depsc L2_SUC;

% [MAL]
figure(205); clf;
axes('position',[0.15 0.15 0.75 0.75]);
box on; hold on;
plot(t((length(t1)+1):end),(MAL_mito((length(t1)+1):end)+MAL_extm((length(t1)+1):end))*1e9,'k','LineWidth',2.0);
p = plot(t_exp, MAL_exp, 'k^','MarkerFaceColor', [0.6 0.6 0.6],'MarkerSize',12);
set(gca,'Fontsize',18);
set(gca,'Ytick',0:500:2000);
set(gca,'Xtick',0:100:500);
% set(gca,'XtickLabel',[]);
xlabel('Time (sec)');
ylabel('[MAL]_{total} (nmoles/mg)');
xlim([0 500]);
ylim([0 2000]);
text((500-0)*0.05,(2000-0)*0.9,'E','FontSize',24);
print -f205 -r600 -depsc L2_MAL;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% Part II: Simulating LaNoue et al.'s experiments with 1mM pyruvate
% defining the minimum concentration allowed for components
xo = default_IC_vitro();

%% Add PI and incubate for 30 seconds
% buffer concentrations
xo(iH_c)    = 10^(-7.2); % LaNoue et al., 1970
xo(iMg_c)   = 5e-3;      % LaNoue et al., 1970
xo(iK_c)    = 150e-3;    % LaNoue et al., 1970
xo(iGLC_c)  = 30e-3;     % LaNoue et al., 1970
% load ASP into the matrix
xo(iGLU_x) = 0;
xo(iGLU_c) = 0;
xo(iASP_c) = 0;
xo(iASP_x) = 5e-3;

% PI and ADP concentrations (state-2)
xo(iPi_c)   = 20e-3;     % LaNoue et al., 1970
xo(iADP_c)  = 0;

% tic
ExpType = 1;
StateType = 2;
t_ib = 30; % incubation time
[t,x] = ode15s(@IsoMito_dXdT,[0 t_ib],xo,options,param,ExpType,StateType);
% toc
xo = x(end,:);
x1 = x;
t1 = t;

%% Adding pyruvate and malate
xo(iPYR_c) = 1e-3;

%% Run the simulation for a long time
indexC = [1 3:63];
options = odeset('MaxStep',5e-1,'NonNegative', indexC);

tic
ExpType = 1;
StateType = 2;
[t,x] = ode15s(@IsoMito_dXdT,[0 250],xo,options,param,ExpType,StateType);
toc
xo_test = x(end,:);
[t_test,x_test] = ode15s(@IsoMito_dXdT,[0 1],xo_test,options,param,ExpType,StateType);
  
x = [x1' x']';
t = [(t1-t_ib)' t']';

%% Computing simulated concentrations
GLU_mito = ( x(:,iGLU_x)*W_x + x(:,iGLU_i)*W_i )*rho_m; 
GLU_extm = x(:,iGLU_c)*W_c*rho_m;

ASP_mito = ( x(:,iASP_x)*W_x + x(:,iASP_i)*W_i )*rho_m; 
ASP_extm = x(:,iASP_c)*W_c*rho_m;

%% Plotting figures
% [GLU] and [ASP]
figure(206); clf;
axes('position',[0.15 0.15 0.75 0.75]);
box on; hold on;
plot(t((length(t1)+1):end),(ASP_mito((length(t1)+1):end)+ASP_extm((length(t1)+1):end))*1e9,'k--','LineWidth',2.0);
p = plot(t_exp_fig3, ASP_exp, 'ko','MarkerFaceColor',[1 1 1],'MarkerSize',12);
plot(t((length(t1)+1):end),(GLU_mito((length(t1)+1):end)+GLU_extm((length(t1)+1):end))*1e9,'k-','LineWidth',2.0);
p = plot(t_exp_fig3, GLU_exp, 'k^','MarkerFaceColor', [0.6 0.6 0.6],'MarkerSize',12);
set(gca,'Fontsize',18);
set(gca,'Ytick',0:5:15);
set(gca,'Xtick',0:50:250);
% set(gca,'XtickLabel',[]);
xlabel('Time (sec)');
ylabel('[GLU]_{total} and [ASP]_{total} (nmoles/mg)');
xlim([0 250]);
ylim([0 15]);
text((500-0)*0.05,(15-0)*0.9,'F','FontSize',24);
print -f206 -r600 -depsc L2_GLUASP;

%% End of simulation
display('done...yeah~');
display(' ');

