% This syntax is used to generate simulations for Bose et al.'s state-3 
% PI-dose curves on isolated rat heart mitochondria (Bose et al., 2003)

% Copyright: 
% Fan Wu and Daniel. A Beard, May 2007
% Computational Engineering Group
% Biotechnology and Bioengineering Center
% Medical College of Wisconsin
 
%% Defining global variables% close all;
clear; clear global;

global iH_x idPsi iATP_x iADP_x iAMP_x iGTP_x iGDP_x iPi_x iNADH_x ...
       iQH2_x iOAA_x iACCOA_x iCIT_x iICIT_x iAKG_x iSCOA_x iCOASH_x ...
       iSUC_x iFUM_x iMAL_x iGLU_x iASP_x iK_x iMg_x iO2_x iCO2tot_x ...
       iCred_i iATP_i iADP_i iAMP_i iPi_i iH_i iMg_i iK_i ...
       iATP_c iADP_c iPi_c iH_c iMg_c iK_c ...
       iPYR_x iPYR_i iPYR_c iCIT_i iCIT_c iAKG_i iAKG_c iSUC_i iSUC_c ...
       iMAL_i iMAL_c iASP_i iASP_c iGLU_i iGLU_c iFUM_i iFUM_c ...
       iICIT_i iICIT_c iGLC_c iG6P_c iPCr_c iAMP_c;

define_global();

%% Setting physical parameters and fixed pools
W_m = 0.664*1.09;   % mitochondrial water space (ml water per ml mito) [VB2002]
W_x = 0.9*W_m;      % Matrix water space, (l mito water) (l mito)^{-1}
W_i = 0.1*W_m;      % IM water space, (l IM water) (l mito)^{-1} 
W_c = 80*10;           % buffer, (l buffer water) (l mito)^{-1}
rho_m = 3.6697e-6;  % (l mito) (mg protein)^{-1}

Ctot   = 2.70e-3;               % M; total cytoC
Qtot   = 1.35e-3;               % M; total Q + QH2
NADtot = 2.97e-3;               % M; total NAD+NADH

% Defining the minimum concentration allowed for components
MinCon = 1e-32;   

%% Loading initial conditions
xo = default_IC_vitro();
xo(iK_x) = 0.125;

%% Loading paramters and experimental data

load param14.mat;

% Loading experimental data curves (from Bose et al., 2003)
load Pi_exp_a; Pi_exp_a = Pi_exp_a/1e3;
load d_Psi_exp_a;
load NADH_exp_a;
load JO2_exp_a;

% Cytochrome C data--percent reduced (from Bose et al., 2003)
CPR_exp_a(1) = 0.084;
CPR_exp_a(2) = 0.171;

%% Running simulations
indexC = [1 3:63];
options = odeset('MaxStep',5e-2,'NonNegative', indexC);

% Setting buffer conditions
xo(iH_c)   = 10^(-7.1);
xo(iMg_c)  = 5e-3;
xo(iK_c)   = 150e-3;
xo(iH_i)   = xo(iH_c);
xo(iMg_i)  = xo(iMg_c);
xo(iK_i)   = xo(iK_c);

% Adding glutamate and malate
xo(iGLU_c) = 5e-3;
xo(iMAL_c) = 5e-3;
xo(iPi_c)  = 0.001e-3;
tic
ExpType = 2;
StateType = 3;
[t,x] = ode15s(@IsoMito_dXdT,[0 60],xo,options,param,ExpType,StateType);
toc
xo = x(end,:);

%% Plotting ready-to-publish curves
tic
for i = 1:1:21
    i

    % Adding ADP into the buffer
    xo(iADP_c) = 1.3e-3;    
    ExpType = 2;
    StateType = 3;
    [t,x] = ode15s(@IsoMito_dXdT,[0 4],xo,options,param,ExpType,StateType);
    xo = x(end,:);
    [t,x] = ode15s(@IsoMito_dXdT,[0 60],xo,options,param,ExpType,StateType);
    xo1 = x(end,:);
    
    % Adding Pi into the buffer   
    Pi_a(i) = (i-1)*0.1/1e3;
    xo1(iPi_c) = Pi_a(i); 
    % Avoiding very low cytoplamic PI concentration
    if xo1(iPi_c) < 0.001e-3
        xo1(iPi_c) = 0.001e-3;
    end
    [t,x] = ode15s(@IsoMito_dXdT,[0 30],xo1,options,param,ExpType,StateType);    


    H_x_PI(i) = x(end,iH_x);
    NADH_x_PI(i) = x(end,iNADH_x);
    Cred_i_PI(i) = x(end,iCred_i);
    dPsi_PI(i) = x(end,idPsi);    
    ATP_x_PI(i) = x(end,iATP_x);
    
    % Computing oxygen consumption rates
    J = fluxes(x(end,:),param,ExpType,StateType);
    JO2_a(i) = (J(3)/2)*1.8957e5;
end
for i = 22:1:41
    i
    % Adding ADP into the buffer
    xo(iADP_c) = 1.3e-3;    
    ExpType = 2;
    StateType = 3;
    [t,x] = ode15s(@IsoMito_dXdT,[0 60],xo,options,param,ExpType,StateType);
    xo1 = x(end,:);
    
    % Adding Pi into the buffer   
    Pi_a(i) = ((i-21)*0.4 + 2)/1e3;
    xo1(iPi_c) = Pi_a(i); 
    % Avoiding very low cytoplamic PI concentration   
    if xo1(iPi_c) < 0.001e-3
        xo1(iPi_c) = 0.001e-3;
    end
    [t,x] = ode15s(@IsoMito_dXdT,[0 30],xo1,options,param,ExpType,StateType);   
    
    H_x_PI(i) = x(end,iH_x);
    NADH_x_PI(i) = x(end,iNADH_x);
    Cred_i_PI(i) = x(end,iCred_i);
    dPsi_PI(i) = x(end,idPsi);    
    ATP_x_PI(i) = x(end,iATP_x);
    
    % Computing oxygen consumption rates
    J = fluxes(x(end,:),param,ExpType,StateType);
    JO2_a(i) = (J(3)/2)*1.8957e5;
end
toc

% Saving results for calling from plot_BoseData.m
save B3PI.mat Pi_a H_x_PI NADH_x_PI Cred_i_PI dPsi_PI JO2_a ATP_x_PI;

display('done...yeah~');
display(' ');



